@@ -22,6 +22,7 @@ import threading
 
 
 from bzrlib import (
+    cethread,
     osutils,
     transport,
     urlutils,
@@ -242,94 +243,15 @@ class TestingChrootServer(chroot.ChrootServer):
         raise NotImplementedError
 
 
-class ThreadWithException(threading.Thread):
-    """A catching exception thread.
-
-    If an exception occurs during the thread execution, it's caught and
-    re-raised when the thread is joined().
-    """
-
-    def __init__(self, *args, **kwargs):
-        # There are cases where the calling thread must wait, yet, if an
-        # exception occurs, the event should be set so the caller is not
-        # blocked. The main example is a calling thread that want to wait for
-        # the called thread to be in a given state before continuing.
-        try:
-            event = kwargs.pop('event')
-        except KeyError:
-            # If the caller didn't pass a specific event, create our own
-            event = threading.Event()
-        super(ThreadWithException, self).__init__(*args, **kwargs)
-        self.set_ready_event(event)
-        self.exception = None
-        self.ignored_exceptions = None # see set_ignored_exceptions
-
-    # compatibility thunk for python-2.4 and python-2.5...
-    if sys.version_info < (2, 6):
-        name = property(threading.Thread.getName, threading.Thread.setName)
-
-    def set_ready_event(self, event):
-        """Set the ``ready`` event used to synchronize exception catching.
-
-        When the thread uses an event to synchronize itself with another thread
-        (setting it when the other thread can wake up from a ``wait`` call),
-        the event must be set after catching an exception or the other thread
-        will hang.
-
-        Some threads require multiple events and should set the relevant one
-        when appropriate.
-        """
-        self.ready = event
-
-    def set_ignored_exceptions(self, ignored):
-        """Declare which exceptions will be ignored.
-
-        :param ignored: Can be either:
-           - None: all exceptions will be raised,
-           - an exception class: the instances of this class will be ignored,
-           - a tuple of exception classes: the instances of any class of the
-             list will be ignored,
-           - a callable: that will be passed the exception object
-             and should return True if the exception should be ignored
-        """
-        if ignored is None:
-            self.ignored_exceptions = None
-        elif isinstance(ignored, (Exception, tuple)):
-            self.ignored_exceptions = lambda e: isinstance(e, ignored)
-        else:
-            self.ignored_exceptions = ignored
-
-    def run(self):
-        """Overrides Thread.run to capture any exception."""
-        self.ready.clear()
-        try:
-            try:
-                super(ThreadWithException, self).run()
-            except:
-                self.exception = sys.exc_info()
-        finally:
-            # Make sure the calling thread is released
-            self.ready.set()
-
+class TestThread(cethread.CatchingExceptionThread):
 
     def join(self, timeout=5):
-        """Overrides Thread.join to raise any exception caught.
-
-
-        Calling join(timeout=0) will raise the caught exception or return None
-        if the thread is still alive.
+        """Overrides to use a default timeout.
 
         The default timeout is set to 5 and should expire only when a thread
         serving a client connection is hung.
         """
-        super(ThreadWithException, self).join(timeout)
-        if self.exception is not None:
-            exc_class, exc_value, exc_tb = self.exception
-            self.exception = None # The exception should be raised only once
-            if (self.ignored_exceptions is None
-                or not self.ignored_exceptions(exc_value)):
-                # Raise non ignored exceptions
-                raise exc_class, exc_value, exc_tb
+        super(TestThread, self).join(timeout)
         if timeout and self.isAlive():
             # The timeout expired without joining the thread, the thread is
             # therefore stucked and that's a failure as far as the test is
@@ -342,13 +264,6 @@ class ThreadWithException(threading.Thread):
             sys.stderr.write('thread %s hung\n' % (self.name,))
             #raise AssertionError('thread %s hung' % (self.name,))
 
-    def pending_exception(self):
-        """Raise the caught exception.
-
-        This does nothing if no exception occurred.
-        """
-        self.join(timeout=0)
-
 
 class TestingTCPServerMixin:
     """Mixin to support running SocketServer.TCPServer in a thread.
@@ -522,8 +437,8 @@ class TestingThreadingTCPServer(TestingTCPServerMixin,
         """Start a new thread to process the request."""
         started = threading.Event()
         stopped = threading.Event()
-        t = ThreadWithException(
-            event=stopped,
+        t = TestThread(
+            sync_event=stopped,
             name='%s -> %s' % (client_address, self.server_address),
             target = self.process_request_thread,
             args = (started, stopped, request, client_address))
@@ -589,8 +504,8 @@ class TestingTCPServerInAThread(transport.Server):
 
     def start_server(self):
         self.server = self.create_server()
-        self._server_thread = ThreadWithException(
-            event=self.server.started,
+        self._server_thread = TestThread(
+            sync_event=self.server.started,
             target=self.run_server)
         self._server_thread.start()
         # Wait for the server thread to start (i.e release the lock)
@@ -606,7 +521,7 @@ class TestingTCPServerInAThread(transport.Server):
         self._server_thread.pending_exception()
         # From now on, we'll use a different event to ensure the server can set
         # its exception
-        self._server_thread.set_ready_event(self.server.stopped)
+        self._server_thread.set_sync_event(self.server.stopped)
 
     def run_server(self):
         self.server.serve()
@@ -694,8 +609,6 @@ class TestingSmartServer(TestingThreadingTCPServer, server.SmartTCPServer):
         server.SmartTCPServer.__init__(self, backing_transport,
                                        root_client_path)
     def serve(self):
-        # FIXME: No test are exercising the hooks for the test server
-        # -- vila 20100618
         self.run_server_started_hooks()
         try:
             TestingThreadingTCPServer.serve(self)
@@ -803,7 +716,3 @@ class ReadonlySmartTCPServer_for_testing_v2_only(
         """Get a backing transport from a server we are decorating."""
         url = 'readonly+' + backing_transport_server.get_url()
         return transport.get_transport(url)
-
-
-
-
